#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <ctype.h>

int check(int n,char ele[],double x1[],double y1[],double z1[],double x2[],double y2[],double z2[]);
double charge(char x);

int main(int argc, char* argv[])
{
    const int n_max=100;
    const double pi=3.1415926;
    const double r_min=2.0;
    const double r_max=16.1;
    const double r_step=0.1;

    int p,t,a,b,g;
    a=atoi(argv[2]);
    b=atoi(argv[3]);
    p=atoi(argv[4]);
    t=atoi(argv[5]);
    g=atoi(argv[6]);
    printf("%d %d %d\n",a,b,g);
    int n;
    double rx[n_max],ry[n_max],rz[n_max],nx[n_max],ny[n_max],nz[n_max];
    char ele[n_max];
    char input[100],hem[100];
    strcpy(hem,argv[1]);
    strcpy(input,hem);
    strcat(input,".xyz");
    FILE* fin=fopen(input,"r");
    fscanf(fin,"%d\n\n",&n);
    printf("%d\n",n);
    for(int i=0;i<n;++i)
	fscanf(fin,"%c%lf%lf%lf\n",ele+i,rx+i,ry+i,rz+i);
    fclose(fin);
    for(int i=0;i<n;++i)
	printf("%d %c\t%15.6lf\t%15.6lf\t%15.6lf\n",i,ele[i],rx[i],ry[i],rz[i]);
    char str[100];
    for(double r=r_min;r<=r_max;r+=r_step){
	double alpha=pi*a/180,beta=pi*b/180;
	double phi=pi*p/180,theta=pi*t/180;
	double gamma=pi*g/180;
	char card[1024]={0};
	char filename[1024]={0};
	card[0]=0;filename[0]=0;
	printf("dimer_%s_%03d_%03d_%03d_%03d_%03d_%04.1lf.gjf\n",hem,a,b,p,t,g,r);
	printf("r= %0.1lf\tphi= %d\ttheta= %d\nalpha= %d\tbeta= %d\tgamma= %d\n",r,p,t,a,b,g);
	sprintf(filename,"dimer_%s_%03d_%03d_%03d_%03d_%03d_%04.1lf.gjf",hem,a,b,p,t,g,r);
	sprintf(card,"r= %0.1lf\tphi= %d\ttheta= %d\nalpha= %d\tbeta= %d\tgamma= %d",r,p,t,a,b,g);
	
	for(int i=0;i<n;++i){
	    double x0=rx[i],y0=ry[i],z0=rz[i];
	    double x1,y1,z1,x2,y2,z2,x,y,z;
	
	    x1=x0*cos(beta)-y0*sin(beta);
	    y1=x0*sin(beta)+y0*cos(beta);
	    z1=z0;
	    //rotate the second molecule through beta degrees along Z axis
	    x2=z1*sin(theta)+x1*cos(theta);
	    y2=y1;
	    z2=z1*cos(theta)-x1*sin(theta);
	    //rotate the second molecule through theta degrees along Y axis
	    x=x2*cos(gamma)-y2*sin(gamma);
	    y=x2*sin(gamma)+y2*cos(gamma);
	    z=z2;
	    //rotate the second molecule through gamma degrees along Z axis
	    nx[i]=x+r*sin(phi)*cos(alpha);
	    ny[i]=y+r*sin(phi)*sin(alpha);
	    nz[i]=z+r*cos(phi);
	    //put the second to spherical coordinate (R, alpha, phi)
	}
	
	switch(check(n,ele,rx,ry,rz,nx,ny,nz)){
	    case -1: printf("Too Close:\n%s\n%s\n",filename,card);
		    continue;
		    break;
	    case 0: break; 
	    case 1: printf("Too Far:\n%s\n%s\n",filename,card);
		    continue;
		    break;
	}
	FILE* fout=fopen(filename,"w");
	printf("%s\n%s\n\n",filename,card);
	
	fprintf(fout,"%%NProcShared=8\n");
	fprintf(fout,"%%Mem=16GB\n");
	fprintf(fout,"#P 6-311+G(d,p) PBE1PBE EmpiricalDispersion=GD3\n");
	fprintf(fout,"\n");
	fprintf(fout,"%s\n",card);
	fprintf(fout,"\n");
	fprintf(fout,"0 1\n");
	for(int i=0;i<n;++i){
	    fprintf(fout,"%c\t%15.6lf\t%15.6lf\t%15.6lf\n",ele[i],rx[i],ry[i],rz[i]);
	    fprintf(fout,"%c\t%15.6lf\t%15.6lf\t%15.6lf\n",ele[i],nx[i],ny[i],nz[i]);}
	fprintf(fout,"\n");
	fprintf(fout,"\n");
	fclose(fout);
    }
    return 0;
}

//This function will check the closest distance of atoms in two molecules. The -1 mean too close, 1 mean too far, and 0 mean the distance is acceptable.
int check(int n,char ele[],double x1[],double y1[],double z1[],double x2[],double y2[],double z2[])
{
    double r[256]={0.0};
    r[(int)'C']=1.72;
    r[(int)'H']=1.10;
    r[(int)'O']=1.40;
    r[(int)'N']=1.50;

    double rmin=300.0;
    double r_int=0.0;
    int val=0;
    for(int i=0;i<n;++i)
        for(int j=0;j<n;++j){
            double rad=(x1[i]-x2[j])*(x1[i]-x2[j])+(y1[i]-y2[j])*(y1[i]-y2[j])+(z1[i]-z2[j])*(z1[i]-z2[j]);
            double r0=r[(int)ele[i]]+r[(int)ele[j]];
	    if(rad<rmin){
                rmin=rad;
                r_int=r0;}
        }
    if(rmin<(r_int-0.5)*(r_int-0.5))val=-1;
    if(rmin>(r_int+0.5)*(r_int+0.5))val=1;
    return val;
}

